@@ -115,6 +115,8 @@ entry_get_string (GConfEntry *entry)
 static gboolean
 parse_binding (Binding *binding)
 {
+        gboolean success;
+
         g_return_val_if_fail (binding != NULL, FALSE);
 
         binding->key.keysym = 0;
@@ -128,10 +130,15 @@ parse_binding (Binding *binding)
                 return FALSE;
         }
 
-        return egg_accelerator_parse_virtual (binding->binding_str,
-                                              &binding->key.keysym,
-                                              &binding->key.keycodes,
-                                              &binding->key.state);
+        success = egg_accelerator_parse_virtual (binding->binding_str,
+                                                 &binding->key.keysym,
+                                                 &binding->key.keycodes,
+                                                 &binding->key.state);
+
+        if (!success)
+            g_warning (_("Key binding (%s) is invalid"), binding->gconf_key);
+
+        return success;
 }
 
 static gint
@@ -220,7 +227,6 @@ bindings_get_entry (GsdKeybindingsManager *manager,
                 if (!tmp_elem)
                         manager->priv->binding_list = g_slist_prepend (manager->priv->binding_list, new_binding);
         } else {
-                g_warning (_("Key binding (%s) is invalid"), gconf_key);
                 g_free (new_binding->binding_str);
                 g_free (new_binding->action);
                 g_free (new_binding->gconf_key);
@@ -559,15 +565,16 @@ gsd_keybindings_manager_start (GsdKeybindingsManager *manager,
 
         client = gconf_client_get_default ();
 
-        manager->priv->allowed_keys = gconf_client_get_list (client,
-                                                             ALLOWED_KEYS_KEY,
-                                                             GCONF_VALUE_STRING,
-                                                             NULL);
         manager->priv->notify = register_config_callback (manager,
                                                           client,
                                                           GCONF_BINDING_DIR,
                                                           (GConfClientNotifyFunc) bindings_callback);
 
+        manager->priv->allowed_keys = gconf_client_get_list (client,
+                                                             ALLOWED_KEYS_KEY,
+                                                             GCONF_VALUE_STRING,
+                                                             NULL);
+
         dpy = gdk_display_get_default ();
         screen_num = gdk_display_get_n_screens (dpy);
 
