@@ -67,6 +67,11 @@
 
 #define VIDEO_KEYSYM    "XF86Display"
 
+/* Number of seconds that the confirmation dialog will last before it resets the
+ * RANDR configuration to its old state.
+ */
+#define CONFIRMATION_DIALOG_SECONDS 30
+
 /* name of the icon files (gsd-xrandr.svg, etc.) */
 #define GSD_XRANDR_ICON_NAME "gsd-xrandr"
 
@@ -109,6 +114,33 @@ G_DEFINE_TYPE (GsdXrandrManager, gsd_xrandr_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
+/* Filters out GNOME_RR_ERROR_NO_MATCHING_CONFIG from
+ * gnome_rr_config_apply_from_filename(), since that is not usually an error.
+ */
+static gboolean
+apply_configuration_from_filename (GsdXrandrManager *manager, const char *filename, GError **error)
+{
+        struct GsdXrandrManagerPrivate *priv = manager->priv;
+        GError *my_error;
+        gboolean success;
+
+        my_error = NULL;
+        success = gnome_rr_config_apply_from_filename (priv->rw_screen, filename, &my_error);
+        if (success)
+                return TRUE;
+
+        if (g_error_matches (my_error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_MATCHING_CONFIG)) {
+                /* This is not an error; the user probably changed his monitors
+                 * and so they don't match any of the stored configurations.
+                 */
+                g_error_free (my_error);
+                return TRUE;
+        }
+
+        g_propagate_error (error, my_error);
+        return FALSE;
+}
+
 static void
 restore_backup_configuration_without_messages (const char *backup_filename, const char *intended_filename)
 {
@@ -119,15 +151,13 @@ restore_backup_configuration_without_messages (const char *backup_filename, cons
 static void
 restore_backup_configuration (GsdXrandrManager *manager, const char *backup_filename, const char *intended_filename)
 {
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
         int saved_errno;
-        char *msg;
 
         if (rename (backup_filename, intended_filename) == 0) {
                 GError *error;
 
                 error = NULL;
-                if (!gnome_rr_config_apply_from_filename (priv->rw_screen, intended_filename, &error)) {
+                if (!apply_configuration_from_filename (manager, intended_filename, &error)) {
                         error_message (manager, _("Could not restore the display's configuration"), error, NULL);
 
                         if (error)
@@ -139,14 +169,25 @@ restore_backup_configuration (GsdXrandrManager *manager, const char *backup_file
 
         saved_errno = errno;
 
-        msg = g_strdup_printf ("Could not rename %s to %s: %s",
-                               backup_filename, intended_filename,
-                               g_strerror (saved_errno));
-        error_message (manager,
-                       _("Could not restore the display's configuration from a backup"),
-                       NULL,
-                       msg);
-        g_free (msg);
+        /* ENOENT means the original file didn't exist.  That is *not* an error;
+         * the backup was not created because there wasn't even an original
+         * monitors.xml (such as on a first-time login).  Note that *here* there
+         * is a "didn't work" monitors.xml, so we must delete that one.
+         */
+        if (saved_errno == ENOENT)
+                unlink (intended_filename);
+        else {
+                char *msg;
+
+                msg = g_strdup_printf ("Could not rename %s to %s: %s",
+                                       backup_filename, intended_filename,
+                                       g_strerror (saved_errno));
+                error_message (manager,
+                               _("Could not restore the display's configuration from a backup"),
+                               NULL,
+                               msg);
+                g_free (msg);
+        }
 
         unlink (backup_filename);
 }
@@ -199,7 +240,7 @@ timeout_response_cb (GtkDialog *dialog, int response_id, gpointer data)
 }
 
 static gboolean
-user_says_things_are_ok (GsdXrandrManager *manager)
+user_says_things_are_ok (GsdXrandrManager *manager, GdkWindow *parent_window)
 {
         TimeoutDialog timeout;
         guint timeout_id;
@@ -212,7 +253,9 @@ user_says_things_are_ok (GsdXrandrManager *manager)
                                                  GTK_MESSAGE_QUESTION,
                                                  GTK_BUTTONS_NONE,
                                                  _("Does the display look OK?"));
-        timeout.countdown = 10;
+
+        timeout.countdown = CONFIRMATION_DIALOG_SECONDS;
+
         print_countdown_text (&timeout);
 
         gtk_dialog_add_button (GTK_DIALOG (timeout.dialog), _("_Restore Previous Configuration"), GTK_RESPONSE_CANCEL);
@@ -223,10 +266,16 @@ user_says_things_are_ok (GsdXrandrManager *manager)
                           G_CALLBACK (timeout_response_cb),
                           &timeout);
 
+        gtk_widget_realize (timeout.dialog);
+
+        if (parent_window)
+                gdk_window_set_transient_for (gtk_widget_get_window (timeout.dialog), parent_window);
+
         gtk_widget_show_all (timeout.dialog);
-        timeout_id = g_timeout_add_seconds (1,
-                                            timeout_cb,
-                                            &timeout);
+        /* We don't use g_timeout_add_seconds() since we actually care that the user sees "real" second ticks in the dialog */
+        timeout_id = g_timeout_add (1000,
+                                    timeout_cb,
+                                    &timeout);
         gtk_main ();
 
         gtk_widget_destroy (timeout.dialog);
@@ -239,9 +288,8 @@ user_says_things_are_ok (GsdXrandrManager *manager)
 }
 
 static gboolean
-try_to_apply_intended_configuration (GsdXrandrManager *manager, GError **error)
+try_to_apply_intended_configuration (GsdXrandrManager *manager, GdkWindow *parent_window, guint32 timestamp, GError **error)
 {
-        struct GsdXrandrManagerPrivate *priv = manager->priv;
         char *backup_filename;
         char *intended_filename;
         gboolean result;
@@ -251,7 +299,7 @@ try_to_apply_intended_configuration (GsdXrandrManager *manager, GError **error)
         backup_filename = gnome_rr_config_get_backup_filename ();
         intended_filename = gnome_rr_config_get_intended_filename ();
 
-        result = gnome_rr_config_apply_from_filename (priv->rw_screen, intended_filename, error);
+        result = apply_configuration_from_filename (manager, intended_filename, error);
         if (!result) {
                 error_message (manager, _("The selected configuration for displays could not be applied"), error ? *error : NULL, NULL);
                 restore_backup_configuration_without_messages (backup_filename, intended_filename);
@@ -260,7 +308,7 @@ try_to_apply_intended_configuration (GsdXrandrManager *manager, GError **error)
 
         /* Confirm with the user */
 
-        if (user_says_things_are_ok (manager))
+        if (user_says_things_are_ok (manager, parent_window))
                 unlink (backup_filename);
         else
                 restore_backup_configuration (manager, backup_filename, intended_filename);
@@ -272,12 +320,35 @@ out:
         return result;
 }
 
-/* DBus method; see gsd-xrandr-manager.xml for the interface definition */
+/* DBus method for org.gnome.SettingsDaemon.XRANDR ApplyConfiguration; see gsd-xrandr-manager.xml for the interface definition */
 static gboolean
 gsd_xrandr_manager_apply_configuration (GsdXrandrManager *manager,
                                         GError          **error)
 {
-        return try_to_apply_intended_configuration (manager, error);
+        return try_to_apply_intended_configuration (manager, NULL, 0, error);
+}
+
+/* DBus method for org.gnome.SettingsDaemon.XRANDR_2 ApplyConfiguration; see gsd-xrandr-manager.xml for the interface definition */
+static gboolean
+gsd_xrandr_manager_2_apply_configuration (GsdXrandrManager *manager,
+                                          long              parent_window_id,
+                                          long              timestamp,
+                                          GError          **error)
+{
+        GdkWindow *parent_window;
+        gboolean result;
+
+        if (parent_window_id != 0)
+                parent_window = gdk_window_foreign_new_for_display (gdk_display_get_default (), parent_window_id);
+        else
+                parent_window = NULL;
+
+        result = try_to_apply_intended_configuration (manager, parent_window, (guint32) timestamp, error);
+
+        if (parent_window)
+                g_object_unref (parent_window);
+
+        return result;
 }
 
 /* We include this after the definition of gsd_xrandr_manager_apply_configuration() so the prototype will already exist */
@@ -541,7 +612,7 @@ sanitize (GPtrArray *array)
         int i;
         GPtrArray *new;
 
-        g_print ("before sanitizing\n");
+        g_debug ("before sanitizing");
 
         for (i = 0; i < array->len; ++i) {
                 if (array->pdata[i]) {
@@ -553,7 +624,7 @@ sanitize (GPtrArray *array)
         /* Remove configurations that are duplicates of
          * configurations earlier in the cycle
          */
-        for (i = 0; i < array->len; ++i) {
+        for (i = 1; i < array->len; ++i) {
                 int j;
 
                 for (j = 0; j < i; ++j) {
@@ -561,7 +632,7 @@ sanitize (GPtrArray *array)
                         GnomeRRConfig *other = array->pdata[i];
 
                         if (this && other && gnome_rr_config_equal (this, other)) {
-                                g_print ("removing duplicate configuration\n");
+                                g_debug ("removing duplicate configuration");
                                 gnome_rr_config_free (this);
                                 array->pdata[j] = NULL;
                                 break;
@@ -598,7 +669,12 @@ sanitize (GPtrArray *array)
                 }
         }
 
-        g_ptr_array_add (new, NULL);
+        if (new->len > 0) {
+                g_ptr_array_add (new, NULL);
+        } else {
+                g_ptr_array_free (new, TRUE);
+                new = NULL;
+        }
 
         g_ptr_array_free (array, TRUE);
 
@@ -611,7 +687,7 @@ generate_fn_f7_configs (GsdXrandrManager *mgr)
         GPtrArray *array = g_ptr_array_new ();
         GnomeRRScreen *screen = mgr->priv->rw_screen;
 
-        g_print ("Generating configurations\n");
+        g_debug ("Generating configurations");
 
         /* Free any existing list of configurations */
         if (mgr->priv->fn_f7_configs) {
@@ -631,12 +707,13 @@ generate_fn_f7_configs (GsdXrandrManager *mgr)
         g_ptr_array_add (array, make_laptop_setup (screen));
         g_ptr_array_add (array, make_other_setup (screen));
         g_ptr_array_add (array, gnome_rr_config_new_stored (screen, NULL)); /* NULL-GError - if this can't read the stored config, no big deal */
-        g_ptr_array_add (array, NULL);
 
         array = sanitize (array);
 
-        mgr->priv->fn_f7_configs = (GnomeRRConfig **)g_ptr_array_free (array, FALSE);
-        mgr->priv->current_fn_f7_config = 0;
+        if (array) {
+                mgr->priv->fn_f7_configs = (GnomeRRConfig **)g_ptr_array_free (array, FALSE);
+                mgr->priv->current_fn_f7_config = 0;
+        }
 }
 
 static void
@@ -1039,6 +1116,31 @@ add_unsupported_rotation_item (GsdXrandrManager *manager)
         gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 }
 
+static void
+ensure_current_configuration_is_saved (void)
+{
+        GnomeRRScreen *rr_screen;
+        GnomeRRConfig *rr_config;
+
+        /* Normally, gnome_rr_config_save() creates a backup file based on the
+         * old monitors.xml.  However, if *that* file didn't exist, there is
+         * nothing from which to create a backup.  So, here we'll save the
+         * current/unchanged configuration and then let our caller call
+         * gnome_rr_config_save() again with the new/changed configuration, so
+         * that there *will* be a backup file in the end.
+         */
+
+        rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), NULL, NULL, NULL); /* NULL-GError */
+        if (!rr_screen)
+                return;
+
+        rr_config = gnome_rr_config_new_current (rr_screen);
+        gnome_rr_config_save (rr_config, NULL); /* NULL-GError */
+
+        gnome_rr_config_free (rr_config);
+        gnome_rr_screen_destroy (rr_screen);
+}
+
 static void
 output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
 {
@@ -1052,6 +1154,8 @@ output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
 	if (!gtk_check_menu_item_get_active (item))
 		return;
 
+        ensure_current_configuration_is_saved ();
+
         output = g_object_get_data (G_OBJECT (item), "output");
         rotation = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "rotation"));
 
@@ -1066,7 +1170,7 @@ output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
                 return;
         }
 
-        try_to_apply_intended_configuration (manager, NULL); /* NULL-GError */
+        try_to_apply_intended_configuration (manager, NULL, 0, NULL); /* NULL-GError */
 }
 
 static void
@@ -1291,14 +1395,9 @@ apply_intended_configuration (GsdXrandrManager *manager, const char *intended_fi
         GError *my_error;
 
         my_error = NULL;
-        if (!gnome_rr_config_apply_from_filename (manager->priv->rw_screen, intended_filename, &my_error)) {
+        if (!apply_configuration_from_filename (manager, intended_filename, &my_error)) {
                 if (my_error) {
-                        if (g_error_matches (my_error, GNOME_RR_ERROR, GNOME_RR_ERROR_NO_MATCHING_CONFIG)) {
-                                /* This is not an error; the user probably
-                                 * changed his monitors and then logged in
-                                 * again, thus restarting gnome-settings-daemon.
-                                 */
-                        } else if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+                        if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                                 error_message (manager, _("Could not apply the stored configuration for monitors"), my_error, NULL);
 
                         g_error_free (my_error);
@@ -1325,7 +1424,7 @@ apply_stored_configuration_at_startup (GsdXrandrManager *manager)
 
         my_error = NULL;
 
-        success = gnome_rr_config_apply_from_filename (manager->priv->rw_screen, backup_filename, &my_error);
+        success = apply_configuration_from_filename (manager, backup_filename, &my_error);
         if (success) {
                 /* The backup configuration existed, and could be applied
                  * successfully, so we must restore it on top of the
